@@ -27,6 +27,7 @@ from pytensor.graph.rewriting.db import (
 from pytensor.link.basic import Linker, PerformLinker
 from pytensor.link.c.basic import CLinker, OpWiseCLinker
 from pytensor.link.jax.linker import JAXLinker
+from pytensor.link.mlx.linker import MLXLinker
 from pytensor.link.numba.linker import NumbaLinker
 from pytensor.link.pytorch.linker import PytorchLinker
 from pytensor.link.vm import VMLinker
@@ -50,6 +51,7 @@ predefined_linkers = {
     "jax": JAXLinker(),
     "pytorch": PytorchLinker(),
     "numba": NumbaLinker(),
+    "mlx": MLXLinker(),
 }
 
 
@@ -67,6 +69,8 @@ exclude = []
 if not config.cxx:
     exclude = ["cxx_only"]
 OPT_NONE = RewriteDatabaseQuery(include=[], exclude=exclude)
+# Minimum set of rewrites needed to evaluate a function. This is needed for graphs with "dummy" Operations
+OPT_MINIMUM = RewriteDatabaseQuery(include=["minimum_compile"], exclude=exclude)
 # Even if multiple merge optimizer call will be there, this shouldn't
 # impact performance.
 OPT_MERGE = RewriteDatabaseQuery(include=["merge"], exclude=exclude)
@@ -77,6 +81,7 @@ OPT_FAST_COMPILE = RewriteDatabaseQuery(include=["fast_compile"], exclude=exclud
 OPT_STABILIZE = RewriteDatabaseQuery(include=["fast_run"], exclude=exclude)
 OPT_STABILIZE.position_cutoff = 1.5000001
 OPT_NONE.name = "OPT_NONE"
+OPT_MINIMUM.name = "OPT_MINIMUM"
 OPT_MERGE.name = "OPT_MERGE"
 OPT_FAST_RUN.name = "OPT_FAST_RUN"
 OPT_FAST_RUN_STABLE.name = "OPT_FAST_RUN_STABLE"
@@ -95,6 +100,7 @@ predefined_optimizers = {
     None: OPT_NONE,
     "None": OPT_NONE,
     "merge": OPT_MERGE,
+    "minimum_compile": OPT_MINIMUM,
     "o4": OPT_FAST_RUN,
     "o3": OPT_O3,
     "o2": OPT_O2,
@@ -138,7 +144,11 @@ class AddDestroyHandler(GraphRewriter):
                 break
         if not supervisor_added:
             warnings.warn(
-                f"A Supervisor feature is missing from {fgraph}.",
+                (
+                    f"A Supervisor feature is missing from {fgraph}.\n"
+                    "This is needed for inplace rewrites. Either exclude inplace rewrites or add a Supervisor feature.\n"
+                    "A Supervisor feature can be added via `pytensor.compile.function.types.add_supervisor_to_fgraph`."
+                ),
                 stacklevel=3,
             )
 
@@ -178,7 +188,7 @@ class PrintCurrentFunctionGraph(GraphRewriter):
     def apply(self, fgraph):
         import pytensor.printing
 
-        print("PrintCurrentFunctionGraph:", self.header)
+        print("PrintCurrentFunctionGraph:", self.header)  # noqa: T201
         pytensor.printing.debugprint(fgraph.outputs)
 
 
@@ -187,6 +197,7 @@ optdb.register(
     "merge1", MergeOptimizer(), "fast_run", "fast_compile", "merge", position=0
 )
 
+
 # After scan1 opt at 0.5 and before ShapeOpt at 1
 # This should only remove nodes.
 # The opt should not do anything that need shape inference.
@@ -398,7 +409,7 @@ class Mode:
             optimizations.
         """
 
-        link, opt = self.get_linker_optimizer(
+        _link, opt = self.get_linker_optimizer(
             self.provided_linker, self.provided_optimizer
         )
         return self.clone(optimizer=opt.register(*optimizations))
@@ -450,6 +461,19 @@ else:
         RewriteDatabaseQuery(include=["fast_run", "py_only"]),
     )
 
+NUMBA = Mode(
+    NumbaLinker(),
+    RewriteDatabaseQuery(
+        include=["fast_run", "numba"],
+        exclude=[
+            "cxx_only",
+            "BlasOpt",
+            "local_careduce_fusion",
+            "scan_save_mem_prealloc",
+        ],
+    ),
+)
+
 JAX = Mode(
     JAXLinker(),
     RewriteDatabaseQuery(
@@ -459,6 +483,10 @@ JAX = Mode(
             "BlasOpt",
             "fusion",
             "inplace",
+            "scan_save_mem_prealloc",
+            # There are specific variants for the LU decompositions supported by JAX
+            "reuse_lu_decomposition_multiple_solves",
+            "scan_split_non_sequence_lu_decomposition_solve",
         ],
     ),
 )
@@ -471,15 +499,24 @@ PYTORCH = Mode(
             "BlasOpt",
             "fusion",
             "inplace",
-            "local_uint_constant_indices",
+            "scan_save_mem_prealloc",
+            "reuse_lu_decomposition_multiple_solves",
+            "scan_split_non_sequence_lu_decomposition_solve",
         ],
     ),
 )
-NUMBA = Mode(
-    NumbaLinker(),
+
+MLX = Mode(
+    MLXLinker(),
     RewriteDatabaseQuery(
-        include=["fast_run", "numba"],
-        exclude=["cxx_only", "BlasOpt", "local_careduce_fusion"],
+        include=["fast_run"],
+        exclude=[
+            "cxx_only",
+            "BlasOpt",
+            "fusion",
+            "inplace",
+            "scan_save_mem_prealloc",
+        ],
     ),
 )
 
@@ -490,9 +527,10 @@ predefined_modes = {
     "JAX": JAX,
     "NUMBA": NUMBA,
     "PYTORCH": PYTORCH,
+    "MLX": MLX,
 }
 
-instantiated_default_mode = None
+_CACHED_RUNTIME_MODES: dict[str, Mode] = {}
 
 
 def get_mode(orig_string):
@@ -500,50 +538,46 @@ def get_mode(orig_string):
         string = config.mode
     else:
         string = orig_string
+
     if not isinstance(string, str):
         return string  # it is hopefully already a mode...
 
-    global instantiated_default_mode
-    # The default mode is cached. However, config.mode can change
-    # If instantiated_default_mode has the right class, use it.
-    if orig_string is None and instantiated_default_mode:
-        if string in predefined_modes:
-            default_mode_class = predefined_modes[string].__class__.__name__
-        else:
-            default_mode_class = string
-        if instantiated_default_mode.__class__.__name__ == default_mode_class:
-            return instantiated_default_mode
+    # Keep the original string for error messages
+    upper_string = string.upper()
 
-    if string in ("Mode", "DebugMode", "NanGuardMode"):
-        if string == "DebugMode":
-            # need to import later to break circular dependency.
-            from .debugmode import DebugMode
+    if upper_string in predefined_modes:
+        return predefined_modes[upper_string]
 
-            # DebugMode use its own linker.
-            ret = DebugMode(optimizer=config.optimizer)
-        elif string == "NanGuardMode":
-            # need to import later to break circular dependency.
-            from .nanguardmode import NanGuardMode
+    global _CACHED_RUNTIME_MODES
+
+    if upper_string in _CACHED_RUNTIME_MODES:
+        return _CACHED_RUNTIME_MODES[upper_string]
+
+    # Need to define the mode for the first time
+    if upper_string == "MODE":
+        ret = Mode(linker=config.linker, optimizer=config.optimizer)
+    elif upper_string in ("DEBUGMODE", "DEBUG_MODE"):
+        from pytensor.compile.debugmode import DebugMode
+
+        # DebugMode use its own linker.
+        ret = DebugMode(optimizer=config.optimizer)
+    elif upper_string == "NANGUARDMODE":
+        from pytensor.compile.nanguardmode import NanGuardMode
+
+        # NanGuardMode use its own linker.
+        ret = NanGuardMode(True, True, True, optimizer=config.optimizer)
 
-            # NanGuardMode use its own linker.
-            ret = NanGuardMode(True, True, True, optimizer=config.optimizer)
-        else:
-            # TODO: Can't we look up the name and invoke it rather than using eval here?
-            ret = eval(string + "(linker=config.linker, optimizer=config.optimizer)")
-    elif string in predefined_modes:
-        ret = predefined_modes[string]
     else:
-        raise Exception(f"No predefined mode exist for string: {string}")
+        raise ValueError(f"No predefined mode exist for string: {string}")
 
-    if orig_string is None:
-        # Build and cache the default mode
-        if config.optimizer_excluding:
-            ret = ret.excluding(*config.optimizer_excluding.split(":"))
-        if config.optimizer_including:
-            ret = ret.including(*config.optimizer_including.split(":"))
-        if config.optimizer_requiring:
-            ret = ret.requiring(*config.optimizer_requiring.split(":"))
-        instantiated_default_mode = ret
+    if config.optimizer_excluding:
+        ret = ret.excluding(*config.optimizer_excluding.split(":"))
+    if config.optimizer_including:
+        ret = ret.including(*config.optimizer_including.split(":"))
+    if config.optimizer_requiring:
+        ret = ret.requiring(*config.optimizer_requiring.split(":"))
+    # Cache the mode for next time
+    _CACHED_RUNTIME_MODES[upper_string] = ret
 
     return ret
 
