@@ -349,6 +349,9 @@ class _tensor_py_operators:
         if (len(pattern) == 1) and (isinstance(pattern[0], list | tuple | np.ndarray)):
             pattern = pattern[0]
         ds_op = pt.elemwise.DimShuffle(input_ndim=self.type.ndim, new_order=pattern)
+        if ds_op.new_order == tuple(range(self.type.ndim)):
+            # No-op
+            return self
         return ds_op(self)
 
     def flatten(self, ndim=1):
@@ -503,7 +506,9 @@ class _tensor_py_operators:
 
         # Check if the number of dimensions isn't too large.
         if self.ndim < index_dim_count:
-            raise IndexError("too many indices for array")
+            raise IndexError(
+                f"too many indices for tensor: tensor is {self.ndim}-dimensional, but {index_dim_count} were indexed"
+            )
 
         # Convert an Ellipsis if provided into an appropriate number of
         # slice(None).
